/*
 * stub for asm/semaphore.h
 */

#ifndef _LINUX_ASM_SEMAPHORE_H
#define _LINUX_ASM_SEMAPHORE_H

#include <linux/wait.h>
#if 1	/* freebsd semaphores */

#include <sys/mutex.h>
struct semaphore {
	struct mtx m;
};

static inline void init_MUTEX(struct semaphore *sem)
{
	mtx_init(&sem->m, "ldev", NULL, MTX_DEF);
}

static inline void down(struct semaphore *sem)
{
	mtx_lock(&sem->m);
}

static inline int down_interruptible(struct semaphore *sem)
{
	down(sem);
	return 0;
}

static inline void up(struct semaphore *sem)
{
	mtx_unlock(&sem->m);
}

#else	/* linux semaphores */
/* from asm/semaphore.h */
struct semaphore {
	atomic_t count;
 	int sleepers;
 	wait_queue_head_t wait;
};

static inline void sema_init (struct semaphore *sem, int val)
{
/*
 *      *sem = (struct semaphore)__SEMAPHORE_INITIALIZER((*sem),val);
 *
 * i'd rather use the more flexible initialization above, but sadly
 * GCC 2.7.2.3 emits a bogus warning. EGCS doesn't. Oh well.
 */
        atomic_set(&sem->count, val);
        sem->sleepers = 0;
        init_waitqueue_head(&sem->wait);
}

static inline void init_MUTEX (struct semaphore *sem)
{
        sema_init(sem, 1);
}

static inline void up(struct semaphore *sem)
{
	// XXX to be filled
}

static inline void down(struct semaphore *sem)
{
	// XXX to be filled
}

static inline int down_interruptible(struct semaphore *sem)
{
	down(sem);
	return 0;
}

/* these are fake implementation to be fixed */
#define spin_lock(sem)	do {} while (0)
#define spin_lock_irq(sem)	do {} while (0)
#define spin_lock_irqsave(sem, flags)	do {flags = 0;} while (0)
#define spin_unlock_irqrestore(sem, flags)	do {} while (0)
#define spin_unlock(sem)	do {} while (0)
#define spin_unlock_irq(sem)	do {} while (0)
#endif
#endif /* _LINUX_ASM_SEMAPHORE_H */
